# This perl script builds a `specs' file for gcc.

# escape blank spaces
sub escblank {
    my ($x) = @_;
    $x =~ s/ /\\ /g;
    return $x;
}

$pkgincludedir=escblank(shift);
$libdir=escblank(shift);
$cilk2cdir=escblank(shift);
$crt0dir=escblank(shift);
$pkgdatadir=escblank(shift);
$pthreadlib=shift;
$gnu_ld=shift;
$cc1=shift;
$cpp=shift;

$lib = '';
$link = '';
$startfile = '';
while (<>) {
  chomp;
  if ($_ eq '*lib:') {
    $lib = (<>);
    chomp $lib; 
  } elsif ($_ eq '*link:') {
    $link = (<>); 
    chomp $link;
  } elsif ($_ eq '*startfile:') {
    $startfile = (<>); 
    chomp $startfile;
  }
}

$ldscript_add="";

if (!($ldscript eq "")) {
  $ldscript_add = "-T${pkgdatadir}$\/${ldscript} ";
}

if ($gnu_ld eq "yes") {
  $rpath=" --rpath ${libdir}";
} else {
  $rpath="";
}

print <<EOF;
*startfile:
${startfile} -L${crt0dir} -lcilkrt0

*lib:
-L${libdir} %{!nostdlib:%{!nodefaultlibs:%{cilk-debug: %{cilk-profile:-lcilk.g.p} %{!cilk-profile:-lcilk.g}} %{!cilk-debug: %{cilk-profile:-lcilk.p} %{!cilk-profile:-lcilk}} ${pthreadlib} %G}} ${lib}

*link:
${ldscript_add}${link}${rpath}

*cilkinclude:
-I${pkgincludedir} 

*cilkcpp:
%(cilkinclude) %{cilk-debug:-DCILK_DEBUG=1} %{cilk-profile:-DCILK_STATS=1 -DCILK_TIMING=1} %{cilk-critical-path:-DCILK_CRITICAL_PATH=1} %{cilk-critical-path:%{!cilk-profile:%e-cilk-critical-path requires -cilk-profile}} -D_REENTRANT

*cilk2c:
-W3 %{Wall:-W5} %{w:-W0} %{cilk-hint:-hint}

*cilkg:
%{g*}

*cilkp:
%{p}

.cilk:
\@cilk

\@cilk:
${cpp} -D__CILK__ %{ansi:-std=c89} %{std*} %{nostdinc*} %{C} %{v} %{A*} %{I*} %{P} %{$} %I %{C:%{!E:%eGNU C does not support -C without using -E}} %{M} %{MM} %{MD:-MD %b.d} %{MMD:-MMD %b.d} %{MG} %{!no-gcc:-D__GNUC__=%v1 -D__GNUC_MINOR__=%v2} %{ansi|std=*:%{!std=gnu*:-trigraphs -D__STRICT_ANSI__}} %{!undef:%{!ansi:%{!std=*:%p}%{std=gnu*:%p}} %P} %{trigraphs} %c %{ffast-math:-D__FAST_MATH__} %{traditional} %{ftraditional:-traditional} %{traditional-cpp:-traditional} %{fleading-underscore} %{fno-leading-underscore} %{W*} %{w} %{pedantic*} %{H} %{d*} %C %{D*} %{U*} %{i*} %Z -D__CILK2C__  %(cilkcpp) -include ${pkgincludedir}/cilk.h %i %{!M:%{!MM:%{!E:%{!pipe:%g.cilk}}}}%{E:%W{o*}}%{M:%W{o*}}%{MM:%W{o*}} |
${cilk2cdir}/cilk2c %(cilk2c) %{!pipe:%g.cilk} %{cc:%W{o*}%{!o*:-o %b.c}}%{!cc:-o %{!pipe:%g.c}} | 
%{!cc:${cpp} -lang-c %{ansi:-std=c89} %(cilkinclude) %{std*} %{nostdinc*} %(cilkcpp) %{traditional} %{ftraditional:-traditional} %{traditional-cpp:-traditional} %{fleading-underscore} %{fno-leading-underscore} %{!pipe:%g.c} %{!pipe:%g.i} |
%{!M:%{!MM:%{!E:${cc1} %{!pipe:%g.i} %1 %{!Q:-quiet} -dumpbase %b.c %{d*} %{m*} %{a*} %(cilkg) %{O*} %{W*} %{w} %{pedantic*} %{std*} %{traditional} %{v:-version} %{pg:-p} %(cilkp) %{f*} %{aux-info*} %{Qn:-fno-ident} %{--help:--help} %{pg:%{fomit-frame-pointer:%e-pg and -fomit-frame-pointer are incompatible}} %{S:%W{o*}%{!o*:-o %b.s}}%{!S:-o %{|!pipe:%g.s}} |
%{!S:as %a %Y %{c:%W{o*}%{!o*:-o %w%b%O}}%{!c:-o %d%w%u%O} %{!pipe:%g.s} %A
}}}}}
EOF

# The specs above come from gcc-2.95, but they should be reasonably
# portable to other systems.
